/*
 *
 * apps/videoServer/videoServer.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/utils/AppUtils.H>

#include <nucleo/network/tcp/TcpServer.H>
#include <nucleo/network/rdv/ServiceAnnouncer.H>

#include "vsConfiguration.H"
#include "vsService.H"

#include <unistd.h>
#include <signal.h>

#include <iostream>
#include <stdexcept>

using namespace nucleo ;
using namespace videoServer ;

int
main(int argc, char **argv) {

  // Configuration file can be specified on command line
  char *CONFIG_FILE = "videoServer.conf" ;
  if (parseCommandLine(argc, argv, "c:", "s", &CONFIG_FILE)<0) {
    std::cerr << std::endl << argv[0] ;
    std::cerr << " [-c configfile]" << std::endl << std::endl ;
    exit(1) ;
  }

  // Create configuration from file
  vsConfiguration config(CONFIG_FILE) ;

  // Create the TCP server
  TcpServer server(config.tcp_port, config.tcp_backlog) ;

  TextRecords photoTextRecords, videoTextRecords ;
  photoTextRecords["path"] = "/grab" ;
  videoTextRecords["path"] = "/push/video" ;
  ServiceAnnouncer::create("videoServer:photo", "_http._tcp.", config.tcp_port, &photoTextRecords) ;
  ServiceAnnouncer::create("videoServer:video", "_http._tcp.", config.tcp_port, &videoTextRecords) ;

  // Wait for clients...
  for (;;) {
    TcpConnection *client = server.waitForNewClient() ;

#ifdef __APPLE__
    // FIXME: why no fork ?
    signal(SIGPIPE, SIG_IGN) ;
#else
    if (!fork()) {
#endif
	 try {
	   vsServe(config, client) ;
	 } catch (std::runtime_error e) {
	   std::cerr << "Runtime error: " << e.what() << std::endl ;
	 } catch (...) {
	   std::cerr << "Exception caught while serving client" << std::endl ;
	 }
	 delete client ;
#ifndef __APPLE__
	 exit(0) ;
    }
#endif

#ifdef SIGCLD
    signal(SIGCLD, SIG_IGN) ;
#else
#  ifdef SIGCHLD
    signal(SIGCHLD, SIG_IGN) ;
#  endif
#endif

    close(client->getFd()) ;
  }

}
