/*
 *
 * apps/videoServer/SubProcess.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "vsSubProcess.H"

#include <nucleo/config.H>

#include <stdio.h>
#include <signal.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>

#include <errno.h>
#include <sys/errno.h>

#include <iostream>
#include <stdexcept>

static void ignoreSignal(int) {}

static const int PIPE_READ=0 ;
static const int PIPE_WRITE=1 ;

namespace videoServer {

  // ------------------------------------------------------------------

  vsSubProcess::vsSubProcess(const char *path) {   
    _nbBytesRead = 0 ;
    pipe(_local) ;
    pipe(_other) ;
    _otherPid = fork() ;
    if (!_otherPid) {
	 close(_local[PIPE_READ]) ;
	 close(_other[PIPE_WRITE]) ;
	 close(0) ; // stdin
	 dup(_other[PIPE_READ]) ; 
	 close(1) ; // stdout
	 dup(_local[PIPE_WRITE]) ; 
	 /* Remove comments to redirect subprocess stderr to stdout
	   close(2) ; // stderr
	   dup(_local[PIPE_WRITE]) ;  
	 */
	 close(_local[PIPE_WRITE]) ;
	 close(_other[PIPE_READ]) ;
	 char *const argv[] = {(char *)path,0} ;
	 execve(path,argv,0) ;
	 // If we are still here after execv, something went wrong...
	 std::cerr << "SubProcess: something went wrong... " ;
	 std::cerr << "Check whether " << path << " is executable" << std::endl ;
    } 
    signal(SIGPIPE, ignoreSignal) ;
    close(_local[PIPE_WRITE]) ;
    // fcntl(_other[PIPE_WRITE], F_SETFD, O_NONBLOCK) ;
    close(_other[PIPE_READ]) ;
    // fcntl(_local[PIPE_READ], F_SETFD, O_NONBLOCK) ;
  }

  vsSubProcess::~vsSubProcess(void) {
    close(_local[PIPE_READ]) ;
    close(_other[PIPE_WRITE]) ;
    /* Remove comments to kill child process
	 if ( kill(_otherPid, SIGINT) != -1 )
	 while (wait(0)!=_otherPid) ;
    */
  }

  void
  vsSubProcess::send(const std::string &s) {
    std::string::size_type nbyte = s.length() ;
    if (nbyte) {
	 int n = write(_other[PIPE_WRITE], s.data(), (int)nbyte) ;
	 if (n != (int)nbyte) {
	   perror("vsSubProcess") ;
	   // std::cerr << "Wrote " << n << " bytes" << std::endl ;
	   throw std::runtime_error("vsSubProcess: unable to send data to subprocess") ;
	 }
    }
  }

  std::string
  vsSubProcess::receive(char *pattern, int secs) {
    struct timeval timeout ;
    timeout.tv_sec = secs ;
    timeout.tv_usec = 0 ;

    std::string result ;

    for(;;) {

	 if (pattern!=0) {
	   std::string::size_type p = _buffer.find(pattern) ;
	   if (p!=std::string::npos) {
		result.assign(_buffer, 0, p) ;
		_buffer.erase(0,p+1) ;
		return result ;
	   }
	 }

	 fd_set rset, wset, eset ;
	 FD_ZERO(&rset) ; FD_SET(_local[PIPE_READ], &rset) ;
	 FD_ZERO(&wset) ;
	 FD_ZERO(&eset) ; FD_SET(_local[PIPE_READ], &eset) ;
#ifdef HAVE_GETDTABLEHI
	 int max = getdtablehi()+1 ;
#else
	 int max = getdtablesize() ;
#endif

	 switch (select(max, &rset, &wset, &eset, (secs>-1) ? &timeout : NULL)) {
	 case -1:
	   throw std::runtime_error("vsSubProcess: select failed (subprocess)") ;
	 case 0:
	   throw std::runtime_error("vsSubProcess: timed out") ;
	 default: {
		char tmpbuffer[512] ;
		int bytesread = read(_local[PIPE_READ], tmpbuffer, 512) ;
		if (bytesread>0) {
		  _nbBytesRead += bytesread ;
		  _buffer.append(tmpbuffer, bytesread) ;
		} else return _buffer ;
	   }
	 }
    }
  }

  // ------------------------------------------------------------------

}
