/*
 *
 * nucleo/core/ReactiveEngine.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/core/ReactiveEngine.H>

#ifdef __APPLE__
#include <nucleo/core/carbon/cReactiveEngine.H>
#endif
#include <nucleo/core/select/sReactiveEngine.H>

#include <iostream>

namespace nucleo {

  // -----------------------------------------------------------------------
  // The native/portable stuff only makes sense on Mac OS for now

#ifdef __APPLE__
  bool ReactiveEngine::_useNativeEngine = true ;
#else
  bool ReactiveEngine::_useNativeEngine = false ;
#endif

  ReactiveEngineImplementation *ReactiveEngine::engine = 0 ;

  bool
  ReactiveEngine::useNativeEngine(bool native) {
    if (!engine) {
#ifdef __APPLE__
	 _useNativeEngine = native ;
#endif
	 createEngine() ;
    }
    return _useNativeEngine ;
  }

  bool
  ReactiveEngine::useNativeEngine(void) {
    return _useNativeEngine ;
  }

  void
  ReactiveEngine::createEngine(void) {
#ifdef __APPLE__
    if (_useNativeEngine) {
	 engine = (ReactiveEngineImplementation*) new cReactiveEngine() ; 
	 // std::cerr << "ReactiveEngine: native implementation started" << std::endl ;
	 return ;
    }
#endif
    engine = (ReactiveEngineImplementation*) new sReactiveEngine() ;
#ifdef __APPLE__
    std::cerr << "ReactiveEngine: portable (select-based) implementation started" << std::endl ;
#endif
  }

  void
  ReactiveEngine::notify(ReactiveObject *notified, Observable *notifier) {
    if (!engine) createEngine() ;
    engine->notify(notified, notifier) ;
  }

  int
  ReactiveEngine::step(long milliseconds) {
    if (!engine) createEngine() ;
    return engine->step(milliseconds) ;
  }

  void
  ReactiveEngine::run(void) {
    if (!engine) createEngine() ;
    engine->run() ;
  }

  void
  ReactiveEngine::stop(void) {
    if (!engine) createEngine() ;
    engine->stop() ;
  }

}
