/*
 *
 * nucleo/core/carbon/cFileKeeper.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/core/carbon/cFileKeeper.H>
#include <nucleo/utils/FileUtils.H>

#include <iostream>

namespace nucleo {
  
  /*
    We sometimes get notified by CF that the socket is readable
    although there is no data available. Moreover, the CF callback
    mechanism doesn't give any clue of the current __state__ of the
    socket (if I receive a kCFSocketReadCallBack, how do I know
    whether it's also writable?).
    
    Until we find something better, we just call select to get the
    full socket state and generate higher-level events if
    necessary.
  */

  // -----------------------------------------------------------------------------

  void
  cFileKeeper::_callback(CFSocketRef s,
					CFSocketCallBackType type,
					CFDataRef address, const void *data, void *info) {
    cFileKeeper *obj = (cFileKeeper *)info ;
    bool doNotify = true ;
    
#if 1
    if (type==kCFSocketReadCallBack) {
	 fd_set rfds, nofds ;
	 FD_ZERO(&rfds) ;
	 FD_ZERO(&nofds) ;
	 FD_SET(obj->_fd, &rfds) ;
	 struct timeval tv ;
	 tv.tv_sec = tv.tv_usec = 0 ;
	 doNotify = select(obj->_fd+1,&rfds,&nofds,&nofds,&tv) ;
    }
#endif

    if (doNotify) obj->notifyObservers() ;
    CFSocketEnableCallBacks(s, type) ;
  }

  void
  cFileKeeper::_cleanup(void) {
    CFRunLoopRemoveSource(CFRunLoopGetCurrent(), _cfRunLoopSource, kCFRunLoopCommonModes);
    CFSocketInvalidate(_cfSocket);
    CFRelease(_cfRunLoopSource);
    CFRelease(_cfSocket);
    _fd = -1 ;
  }

  void
  cFileKeeper::setup(int fd, eventMask mask) {
    if (_fd!=-1) _cleanup() ;

    FileKeeper::setup(fd, mask) ;

    _flags = kCFSocketNoCallBack ;
    if (mask&FileKeeper::R) _flags = _flags | kCFSocketReadCallBack ;
    if (mask&FileKeeper::W) _flags = _flags | kCFSocketWriteCallBack ;

    CFSocketContext context={0,(void *)this,NULL,NULL,NULL};

    _cfSocket = CFSocketCreateWithNative(NULL, fd, _flags, _callback, &context) ;

    if(!_cfSocket) {
	 fd = -1 ;
	 return ;
    }

    // turn off kCFSocketAutomaticallyReenableRead/WriteCallBack and
    // kCFSocketCloseNativeOnInvalidate
    CFSocketSetSocketFlags(_cfSocket, 0) ;

    _cfRunLoopSource = CFSocketCreateRunLoopSource(NULL, _cfSocket, 0);
    if(!_cfRunLoopSource) {
	 fd = -1 ;
	 CFRelease(_cfSocket) ;
	 return ;
    }

    CFRunLoopAddSource(CFRunLoopGetCurrent(), _cfRunLoopSource, kCFRunLoopDefaultMode);
  }

  cFileKeeper::cFileKeeper(void) {
  }

  cFileKeeper::~cFileKeeper(void) {
    if (_fd!=-1) _cleanup() ;
  }

  // -----------------------------------------------------------------------------

}
