/*
 *
 * nucleo/gl/window/glWindow.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/gl/window/glWindow.H>

#if HAVE_AGL
#include <nucleo/gl/window/glWindow_AGL.H>
#endif

#if HAVE_GLX
#include <nucleo/gl/window/glWindow_GLX.H>
#endif

namespace nucleo {

  const char * const glWindow::event::strings[] = {
#define EVENTDEF(eventname,eventnum) #eventname,
#include <nucleo/gl/window/_events.H>
#undef EVENTDEF
  } ;

  glWindow::event::event(void) {
    keystr = "<?>" ;
  }

  glWindow::event&
  glWindow::event::operator = (glWindow::event& src) {
    if (&src!=this) {
	 type = src.type ;
	 width = src.width ;
	 height = src.height ;
	 x = src.x ;
	 y = src.y ;
	 button = src.button ;
	 axis = src.axis ;
	 delta = src.delta ;
	 time = src.time;
	 keysym = src.keysym ;
	 keystr = src.keystr ;
    }
    return *this ;
  }

  void
  glWindow::event::debug(std::ostream& out) const {
    out << "[" ;
    switch (type) {
#define EVENTDEF(eventname,eventnum) case eventnum: out << #eventname ; break ;
#include <nucleo/gl/window/_events.H>
#undef EVENTDEF
    }
    switch (type) {
    case configure:
	 out << " x=" << x << " y=" << y << " w=" << width << " h=" << height ;
	 break ;
    case expose:
    case destroy:
    case enter:
    case leave:
	 break ;
    case keyPress:
    case keyRelease:
	 out << " sym=" << keysym << " str=" << keystr ;
	 break ;
    case buttonPress: 
    case buttonRelease:
	 out << " x=" << x << " y=" << y << " b=" << button ;
	 break ;
    case pointerMotion:
	 out << " x=" << x << " y=" << y ;
	 break ;
    case wheelMotion:
	 out << " a=" << axis << " d=" << delta ;
	 break ;
    }
    out << "]" << std::flush ;
  }

  glWindow::event::~event() {
  }

  // ------------------------------------------------------------------

  void
  glWindow::setGeometry(char *geometry) {
    // [=][<width>{xX}<height>][{+-}<xoffset>{+-}<yoffset>]

    char *ptr = geometry ;
    std::string number ;
    unsigned int width=0, height=0 ;
    int x=0, y=0, sign=1 ;

    getGeometry(&width, &height, &x, &y) ;

    if (*ptr=='=') ptr++ ;
    if (*ptr=='+' || *ptr=='-'
	   ) goto position ;

    while (*ptr!='x' && *ptr!='X') {
	 if (*ptr=='\0') goto abort ;
	 number = number + (*ptr++) ;
    }
    width = (unsigned int) atoi(number.c_str()) ;

    number = "" ; ptr++ ; // skip the x (or X)

    while (*ptr!='\0' && *ptr!='+' && *ptr!='-'
		 ) 
	 number = number + (*ptr++) ;
    height = (unsigned int) atoi(number.c_str()) ;
    if (*ptr=='\0') goto end ;

  position:

    sign = (*ptr=='-') ? -1 : 1 ;
    number = "" ; ptr++ ; // skip the sign

    while (*ptr!='+' && *ptr!='-'
		 ) {
	 if (*ptr=='\0') goto abort ;
	 number = number + (*ptr++) ;
    }
    x = sign * atoi(number.c_str()) ;
    
    sign = (*ptr=='-') ? -1 : 1 ;
    number = "" ; ptr++ ; // skip the sign
    while (*ptr!='\0')
	 number = number + (*ptr++) ;
    y = sign * atoi(number.c_str()) ;

  end:
    //std::cerr << "geometry: " << width << "x" << height << " " << x << "," << y << std::endl ;
    setGeometry(width, height, x, y) ;

  abort:
    return ;
  }

  void
  glWindow::resize(unsigned int newwidth, unsigned int newheight) {
    unsigned int width, height ;
    int x, y ;
    getGeometry(&width, &height, &x, &y) ;
    setGeometry(newwidth, newheight, x, y) ;
  }

  void
  glWindow::move(int newx, int newy) {
    unsigned int width, height ;
    int x, y ;
    getGeometry(&width, &height, &x, &y) ;
    setGeometry(width, height, newx, newy) ;
  }

  void
  glWindow::moveRel(int dx, int dy) {
    unsigned int width, height ;
    int x, y ;
    getGeometry(&width, &height, &x, &y) ;
    setGeometry(width, height, x+dx, y+dy) ;
  }

  // ------------------------------------------------------------------

  glWindow *
  glWindow::create(long options, long eventmask) {
#if HAVE_AGL
    return new glWindow_AGL(options,eventmask) ;
#elif HAVE_GLX
    return new glWindow_GLX(options, eventmask) ;
#endif
    return (glWindow*)0 ;
  }

  // ------------------------------------------------------------------

}
