#ifndef n_glWindow_AGL_H
#define n_glWindow_AGL_H

/*
 *
 * nucleo/gl/window/glWindow_AGL.H --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/utils/FileUtils.H>

#include <nucleo/gl/window/glWindow.H>

#include <Carbon/Carbon.h>
#include <AGL/agl.h>

#include <iostream>
#include <queue>

namespace nucleo {

  class glWindow_AGL : public glWindow {

  protected:

    WindowPtr _window;
    AGLContext _glContext;

    bool _mapped ; // whether the window is currently mapped on-screen
 
    double _aspect ;
    long _options, _eventmask;
    Ptr _fullscreenRestoreState;    
    Rect _rect; // rect is used to translate mouse coordinate in window space

    UInt32 _modifiers ;

    EventTypeSpec _macEvents[24] ;
    unsigned int _nbMacEvents ;
    std::queue<event *> _equeue ;
    static OSStatus _windowEventHandler(EventHandlerCallRef, EventRef, void*) ;

    void setup(long options, long eventmask) ;

    void react(Observable *) ;

  public:

    unsigned int getScreenWidth(void) {
      return (**GetMainDevice()).gdRect.right ;
    }
    unsigned int getScreenHeight(void) {
      return (**GetMainDevice()).gdRect.bottom ;
    }

    glWindow_AGL(long options, long eventmask) ;
    ~glWindow_AGL(void) ;

    void map(void) ;
    void unmap(void) ;

    void setTitle(const char *title) ;
    void getGeometry(unsigned int *width, unsigned int *height, int *x, int *y) ;
    void setGeometry(unsigned int width, unsigned int height, int x, int y) ;
    void fullScreenMode(bool activate) ;
    void setAspectRatio(int width, int height) ;

    void makeCurrent(void) {
      map() ;
      aglSetCurrentContext(_glContext) ;      
    }  
    void swapBuffers(void) {
	 aglSwapBuffers(_glContext) ;
    }

    void showCursor(bool activate) ;
    void warpCursor(int x, int y) ;
    void syncToVbl(int nb_vbls=1) ;
    void raise(void) ;

    bool getNextEvent(event *e) ;
  } ;

}

#endif
