/*
 *
 * nucleo/image/ImageSink.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/image/sink/ImageSink.H>

#ifdef __APPLE__
#include <nucleo/image/sink/qtMovImageSink.H>
#endif

#include <nucleo/image/sink/imagefileImageSink.H>
#include <nucleo/image/sink/serverpushImageSink.H>

#include <nucleo/image/sink/glwindowImageSink.H>

#include <nucleo/image/sink/nudpImageSink.H>
#include <nucleo/image/sink/nudppImageSink.H>
#include <nucleo/image/sink/nserverImageSink.H>

#include <nucleo/core/PluginManager.H>

// #include <nucleo/core/URL.H>
// #include <nucleo/utils/FileUtils.H>

#include <stdexcept>

namespace nucleo {

  // ---------------------------------------------------------------------------

  ImageSink *
  ImageSink::create(const char *u) {
    if (!u) throw std::runtime_error("Can't create an ImageSink from an empty URL...") ;

    URL url(u) ;
    std::string scheme = url.getScheme() ;

    // -------------------------------------------------------

    std::string tag ;

    if ((scheme=="file") || (scheme=="")) {
	 std::string filename = url.isOpaque() ? url.getOpaque() : url.getPath() ;
	 const char *extension = getExtension(filename.c_str()) ;
	 if (!extension) throw std::runtime_error("createImageSink: unknown file type '"+filename+"'") ;
	 cistring iext = extension ;
	 if ((iext == ".vss") || (iext == ".nsp")) return new serverpushImageSink(url) ;
	 if (iext == ".jpg") return new imagefileImageSink(url, Image::JPEG) ;
	 if (iext == ".jpeg") return new imagefileImageSink(url, Image::JPEG) ;
	 if (iext == ".png") return new imagefileImageSink(url, Image::PNG) ;
	 if (iext == ".pam") return new imagefileImageSink(url, Image::PAM) ;
#ifdef __APPLE__
	 if (iext == ".mov") return new qtMovImageSink(url) ;
#endif
	 tag = std::string("extension=")+extension ;
    } else if (scheme=="nudp")
	 return new nudpImageSink(url) ;
    else if ((scheme=="nudpp") || (scheme=="nudp+"))
	 return new nudppImageSink(url) ;
#if HAVE_OPENGL
    else if (scheme=="glwindow") return new glwindowImageSink(url) ;
#endif
    else if (scheme=="nserver") return new nserverImageSink(url) ;
    else {
	 tag = std::string("scheme=")+scheme ;
    }

    ImageSinkFactory factory = (ImageSinkFactory) PluginManager::getSymbol("ImageSink::create",tag) ;
    return (ImageSink *) (*factory) (&url) ;
  }

  // ---------------------------------------------------------------------------

}
