/*
 *
 * nucleo/image/sink/nudpImageSink.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <unistd.h>

#include <nucleo/image/sink/nudpImageSink.H>
#include <nucleo/image/encoding/Conversion.H>

namespace nucleo {

  nudpImageSink::nudpImageSink(const URL &url) {
    std::string hostOrGroup = url.getHost() ; 
    std::string port = url.getPort() ;
    std::string query = url.getQuery() ;
    
    int iport = atoi(port.c_str()) ;
    _udp = new UdpSender(hostOrGroup.c_str(), iport) ;

    std::string encoding ;
    _encoding = Image::JPEG ;    
    if (URL::getQueryArg(query, "encoding", &encoding))
	 _encoding = Image::getEncodingByName(encoding) ;
    
    _quality = 80 ;
    URL::getQueryArg(query, "quality", &_quality) ;

    unsigned int ttl ;
    if (URL::getQueryArg(query, "ttl", &ttl))
	 _udp->setMulticastTTL(ttl) ;
    
    _active = false ;
  }

  nudpImageSink::nudpImageSink(const char *hostOrGroup, int port,
						 Image::Encoding encoding, unsigned int quality,
						 unsigned int ttl) {
    _udp = new UdpSender(hostOrGroup, port) ;
    _encoding = encoding ;
    _quality = quality ;
    if (ttl) _udp->setMulticastTTL(ttl) ;
    _active = false ;
  }

  nudpImageSink::~nudpImageSink(void) {
    delete _udp ;
  }

  bool
  nudpImageSink::start(void) {
    _active = true ;
    return true ;
  }

  bool
  nudpImageSink::stop(void) {
    _active = false ;
    return true ;
  }

  bool
  nudpImageSink::isActive(void) {
    return _active ;
  }

  bool
  nudpImageSink::handle(Image *img) {
    Image copy(*img) ;
    if (!convertImage(&copy, _encoding, _quality)) return false ;
    // std::cerr << "sending " << copy.getSize() << " bytes" << std::endl ;
    if (_udp->send(copy.getData(), copy.getSize())<=0) return false ;
    _chrono.tick() ;
    return true ;
  }
    
}
