/*
 *
 * nucleo/image/source/nudpImageSource.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/image/source/nudpImageSource.H>

#include <stdexcept>

namespace nucleo {

  // ------------------------------------------------------------------

  nudpImageSource::nudpImageSource(Image::Encoding target_encoding,
							const URL &url) {
    _hostOrGroup = url.getHost() ; 
    std::string port = url.getPort() ;
    _port = atoi(port.c_str()) ;

    std::string query = url.getQuery() ;

    std::string encoding ;
    _source_encoding = Image::JPEG ;    
    if (URL::getQueryArg(query, "encoding", &encoding))
	 _source_encoding = Image::getEncodingByName(encoding) ;
    
    _quality = 100 ;
    URL::getQueryArg(query, "quality", &_quality) ;

    _target_encoding = target_encoding==Image::PREFERRED ? _source_encoding : target_encoding ;

    _state = CLOSED ;
  }

  bool
  nudpImageSource::start(void) {
    if (_state!=CLOSED) return false ;

    try {
	 if (_hostOrGroup=="localhost")
	   _udp = new UdpReceiver(_port) ;
	 else
	   _udp = new UdpReceiver(_hostOrGroup.c_str(), _port) ;

	 subscribeTo(_udp) ;

	 _state = OPEN ;
	 _chrono.start() ;
	 return true ;
    } catch (std::runtime_error e) {
	 std::cerr << "nudpImageSource: " << e.what() << std::endl ;
    } catch(...) {
    }

    delete _udp ;
    return false ;
  }

  void
  nudpImageSource::react(Observable *obs) {
    if (obs==_udp) {
	 if (_udp->receive(&_data, &_dataSize)) {
	   // std::cerr << "received " << _dataSize << " bytes" << std::endl ;
	   _state = READY ;
	   notifyObservers() ;
	 }
    }
  }

  bool
  nudpImageSource::getNextImage(Image *img) {
    if (_state!=READY) return false ; 

    img->setEncoding(_source_encoding) ;
    img->setData(_data, _dataSize, Image::DELETE) ;
    _state = OPEN ;
    if (!convertImage(img, _target_encoding, _quality)) return false ;
    _chrono.tick() ;
    return true ;
  }

  bool
  nudpImageSource::stop(void) {
    if (_state==CLOSED) return false ;

    _chrono.stop() ;
    unsubscribeFrom(_udp) ;
    delete _udp ;
    _state = CLOSED ;

    return true ;
  }

  // ------------------------------------------------------------------

}
