/*
 *
 * nucleo/image/source/qtMovImageSource.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/config.H>

#include <nucleo/image/source/qtMovImageSource.H>
#include <nucleo/image/encoding/Conversion.H>

#include <stdio.h>

// --------------------------------------------------------------------------

extern "C" {
  OSErr NativePathNameToFSSpec(char *, FSSpec *, long) ;
}

namespace nucleo {

  // --------------------------------------------------------------------------

  pascal OSErr
  DrawCompleteProc(Movie theMovie, long ptr) {
    qtMovImageSource *obj = (qtMovImageSource*)ptr ;
    obj->_haveImage = true ;
    // std::cerr << "notifyObservers" << std::endl ;
    obj->notifyObservers() ;
    return noErr;
  }

  qtMovImageSource::qtMovImageSource(Image::Encoding e, const URL &u) {
    std::string filename = u.isOpaque() ? u.getOpaque() : u.getPath() ;
    _state = BEZERK ;
    _haveImage = false ;

    EnterMovies() ;

    FSSpec theFSSpec ;
    OSErr result = NativePathNameToFSSpec((char *)filename.c_str(),
								  &theFSSpec, 0 /* flags */) ;
    if (result) {
	 std::cerr << "qtMovImageSource: NativePathNameToFSSpec failed on " << filename << std::endl ;
	 return ;
    }

    short resRefNum = -1;
    result = OpenMovieFile(&theFSSpec, &resRefNum, 0) ;
    if (result) {
	 std::cerr << "qtMovImageSource: OpenMovieFile failed" << std::endl ;
	 return ;
    }
		
    short actualResId = DoTheRightThing ;
    result = NewMovieFromFile(&_theMovie, resRefNum, &actualResId, (unsigned char *) 0, 0, (Boolean *) 0);
    if (result) {
	 std::cerr << "qtMovImageSource: NewMovieFromFile" << std::endl ; 
	 return ;
    }

    if (resRefNum!=-1) CloseMovieFile(resRefNum) ;

    Rect bounds ;
    GetMovieBox(_theMovie, &bounds) ;

    unsigned int width = bounds.right - bounds.left ;
    unsigned int height = bounds.bottom - bounds.top ;

    // k24RGBPixelFormat is MUCH slower
    OSType pixelFormat = k32ARGBPixelFormat ;
    _nucleo_encoding = (e==Image::PREFERRED) ? Image::ARGB : e ;
    _qt_encoding = Image::ARGB ;

    // std::cerr << Image::getEncodingName(_qt_encoding) << " --> " << Image::getEncodingName(_nucleo_encoding) << std::endl ;

    _img.prepareFor(width, height, _qt_encoding) ;
    GWorldPtr offWorld ;
    QTNewGWorldFromPtr(&offWorld, pixelFormat, &bounds, NULL, NULL, 0,
				   _img.getData(), (long)(_img.getBytesPerPixel(_qt_encoding)*width)) ;

    SetGWorld(offWorld, NULL) ;

    _thePlayer = NewMovieController(_theMovie, &bounds, mcTopLeftMovie|mcNotVisible) ;
    SetMovieGWorld(_theMovie, offWorld, NULL);

    std::string query = u.getQuery() ;

    if (URL::getQueryArg(query, "delta", &_deltat))
	 std::cerr << "delta = " << _deltat << std::endl ;
    else
	 _deltat = 10 ;

    if (!URL::getQueryArg(query, "quality", &_quality))
	 _quality = 100 ;

    _state = INACTIVE ;
  }

  bool
  qtMovImageSource::start(void) {
    if (_state!=INACTIVE) return false ;

    SetMovieActive(_theMovie, true);
    _proc = NewMovieDrawingCompleteUPP(DrawCompleteProc);
    SetMovieDrawingCompleteProc(_theMovie, movieDrawingCallWhenChanged,
						  _proc, (long)this); 
    MCDoAction(_thePlayer, mcActionPrerollAndPlay, (void*)Long2Fix(1));

    _state = ACTIVE ;
    _tk = TimeKeeper::create(_deltat*TimeKeeper::millisecond, true) ;
    subscribeTo(_tk) ;
    _chrono.start() ;

    return true ;
  }

  bool
  qtMovImageSource::getNextImage(Image *img) {
    if (_state!=ACTIVE || !_haveImage) return false ;
    img->linkDataFrom(_img) ;
    convertImage(img, _nucleo_encoding, _quality) ;
    _chrono.tick() ;
    _haveImage = false ;
    return true ;
  }

  void
  qtMovImageSource::react(Observable*) {
    if (_state!=ACTIVE) return ;
    // std::cerr << "MCIdle" << std::endl ;
    MCIdle(_thePlayer) ;
  }

  qtMovImageSource::~qtMovImageSource() {
    stop() ;
  }

  bool
  qtMovImageSource::stop() {
    if (_state==ACTIVE) {
	 DisposeMovieController(_thePlayer) ;
	 DisposeMovie(_theMovie) ;
	 DisposeMovieDrawingCompleteUPP(_proc) ;
	 // FIXME: Should call DisposeGWorld(...) ; 
	 _state = BEZERK ; // FIXME: could it be ACTIVE again ?
	 unsubscribeFrom(_tk) ;
	 delete _tk ;
	 return true ;
    }
    return false ;
  }

  // --------------------------------------------------------------------------

}
