/*
 *
 * nucleo/network/rdv/ServiceAnnouncer.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/network/rdv/ServiceAnnouncer.H>

#include <nucleo/core/PluginManager.H>

#include <iostream>
#include <stdexcept>

namespace nucleo {

  class dummyServiceAnnouncer : public ServiceAnnouncer {

  protected:

    std::string name ;

  public:

    dummyServiceAnnouncer(const char *n, const char *type, int port,
					 TextRecords *textRecords,
					 bool autorename,
					 const char *domain, const char *host) {
	 name = n ;
	 std::cerr << "Service '" << n << "' (" << type << ") is running on port " << port ;
	 if (host) std::cerr << " on machine '" << host << "'" ;
	 else std::cerr << " on localhost" << std::endl ;
    }

    state getState(void) { return UNKNOWN ; }
    bool changeName(const char *newname) { return false ; }
    std::string getName(void) { return name ; }

  } ;

  ServiceAnnouncer *
  ServiceAnnouncer::create(const char *name, const char *type, int port,
					  TextRecords *textRecords,
					  bool autorename,
					  const char *domain, const char *host) {
    try {
	 ServiceAnnouncerFactory factory = (ServiceAnnouncerFactory) PluginManager::getSymbol("ServiceAnnouncer::create","*") ;
	 if (factory)
	   return (ServiceAnnouncer*) ((*factory)(name,type,port,textRecords,autorename,domain,host)) ;
    } catch (std::runtime_error e) {
	 std::cerr << e.what() << std::endl ;
    }

    return new dummyServiceAnnouncer(name,type,port,textRecords,autorename,domain,host) ;
  }

}
