/*
 *
 * nucleo/plugins/ffmpeg/ffmpegImageSink.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

// FIXME: ffmpegImageSink is broken.
//  TODO: Check output_example.c in FFmpeg source tree.

#include <nucleo/config.H>

#include <nucleo/plugins/ffmpeg/ffmpegImageSink.H>

#include <nucleo/utils/FileUtils.H>
#include <nucleo/image/encoding/Conversion.H>
#include <nucleo/image/processing/basic/PaintingAndTransforming.H>

#include <unistd.h>

#include <stdexcept>

// ------------------------------------------------------------------------------------------------------

extern "C" {

  void *ffmpegImageSink_factory(const URL *url) {
#if 0
    std::string filename = url->isOpaque() ? url->getOpaque() : url->getPath() ;
    const char *extension = getExtension(filename.c_str()) ;
    if (!strcmp(extension,".mpg"))  return new ffmpegImageSink(*url,"mpeg1video") ;
    if (!strcmp(extension,".mpeg")) return new ffmpegImageSink(*url,"mpeg1video") ;
    if (!strcmp(extension,".swf"))  return new ffmpegImageSink(*url,"swf") ;
#if 0
    // FIXME: These three don't work anymore...
    if (!strcmp(extension,".rm"))   return new ffmpegImageSink(*url,"rm") ;
    if (!strcmp(extension,".avi"))  return new ffmpegImageSink(*url,"avi") ;
    if (!strcmp(extension,".asf"))  return new ffmpegImageSink(*url,"asf") ;
#endif
    throw std::runtime_error("ffmpegImageSink: unknown extension") ;
#else
    return new ffmpegImageSink(*url) ;
#endif
  }

}

// ------------------------------------------------------------------------------------------------------

static void
ffmpegImageSink_write_packet(void *opaque, uint8_t *buf, int size) {
  // std::cerr << "[" << size << "]" << std::flush ;
  int *fd = (int*)opaque ;
  write(*fd,buf,size) ;
}

// ------------------------------------------------------------------------------------------------------

ffmpegImageSink::ffmpegImageSink(int fd, const char* format,
						   int framerate, int bitrate, int gopsize, int qscale) {
  _active = false ;

  _fd = fd ;
  _ownfd = false ;

  _format = format ;
  _framerate = framerate ;
  _bitrate = bitrate ;
  _gopsize = gopsize ;
  _qscale =qscale ;
}

ffmpegImageSink::ffmpegImageSink(const URL &url) {
  _active = false ;

  _filename = url.isOpaque() ? url.getOpaque() : url.getPath() ;
  _fd = createFile(_filename.c_str()) ;
  if (_fd==-1) throw std::runtime_error("ffmpegImageSink: can't create file "+_filename) ;
  _ownfd = true ;
  _format = "" ;

  std::string query = url.getQuery() ;
  std::string arg = URL::getQueryArg(query, "framerate") ;
  _framerate = (arg!="") ? atoi(arg.c_str()) : 25 ;
  arg = URL::getQueryArg(query, "bitrate") ;
  _bitrate = (arg!="") ? atoi(arg.c_str()) : 256000 ;
  arg = URL::getQueryArg(query, "gopsize") ;
  _gopsize = (arg!="") ? atoi(arg.c_str()) : 12 ;
  arg = URL::getQueryArg(query, "qscale") ;
  _qscale = (arg!="") ? atoi(arg.c_str()) : 2 ;
}

// ------------------------------------------------------------------------------------------------------

bool
ffmpegImageSink::init(Image *img) {
  if (!first_oformat) {
    av_register_all() ;
    avcodec_init();
    avcodec_register_all() ;
  }

  if (_format=="")
    _avf = guess_format(0,_filename.c_str(),0) ;
  else
    _avf = guess_format(_format.c_str(),0,0) ;

  if (!_avf) {
    std::cerr << "ffmpegImageSink: couldn't guess format from (" << _format << "," << _filename << ")" << std::endl ;
    return false ;
  }
  if (_avf->video_codec == CODEC_ID_NONE) {
    std::cerr << "ffmpegImageSink: '" << _avf->long_name << "' has no video codec ID" << std::endl ;
    return false ;
  }

  _st.codec.codec = avcodec_find_encoder(_avf->video_codec) ;
  if (!_st.codec.codec) {
    std::cerr << "ffmpegImageSink: avcodec_find_encoder returned 0" << std::endl ;
    return false ;
  }

  if (avcodec_open(&(_st.codec), _st.codec.codec) < 0) {
    std::cerr << "ffmpegImageSink: avcodec_open failed" << std::endl ;
    return false ;
  }
  
  std::cerr << "Found codec: " << _st.codec.codec_name << std::endl ;

  avcodec_get_context_defaults(&(_st.codec)) ;
  _st.codec.width = img->getWidth() ;
  _st.codec.height = img->getHeight() ;
  _st.codec.bit_rate = _bitrate ;
  _st.codec.frame_rate = _framerate ;
  _st.codec.frame_rate_base = 1 ;
  _st.codec.gop_size = _gopsize ;
  _st.codec.flags = 0 ;
  _st.codec.pix_fmt = PIX_FMT_YUV420P ;
  if (_qscale) {
    _st.codec.flags |= CODEC_FLAG_QSCALE ;
    // _st.codec.quality = _qscale ;
    _st.quality = _qscale ;
  }

  _avfc.oformat = _avf ;
  _avfc.nb_streams = 1 ;
  _avfc.streams[0] = &_st ;
  if (init_put_byte(&(_avfc.pb), _inbuffer, sizeof(_inbuffer),
				1, (void *)&_fd,
				0, ffmpegImageSink_write_packet, 0) < 0) {
    std::cerr << "ffmpegImageSink: init_put_byte failed" << std::endl ;
    return false ;
  }
  _avfc.pb.max_packet_size = sizeof(_inbuffer) ;

  if (av_set_parameters(&_avfc, NULL) < 0) {
    std::cerr << "ffmpegImageSink: Invalid output format parameters" << std::endl ;
    return false ;
  }

  // Display some FFmpeg debug information
  dump_format(&_avfc, 0, _filename.c_str(), 1) ;

  _avfc.oformat->write_header(&_avfc) ;

  return true ;
}

bool
ffmpegImageSink::start(void) {
  if (_fd==-1) return false ;

  _active = true ;
  _chrono.start() ;
  return true ;
}

bool
ffmpegImageSink::handle(Image *img) {
  Image copy(*img) ;
  if (!convertImage(&copy, Image::YpCbCr420)) return false ;

  int w = copy.getWidth(), h=copy.getHeight() ;
  if (w&0xF || h&0xF) {
    w = copy.getWidth()&0xFFFFFFF0 ;
    h = copy.getHeight()&0xFFFFFFF0 ;
    // This is weird. Image must have been YpCbCr420 before
    // convertImage (convertImage crops the image dimensions to
    // multiples of 16)
    std::cerr << "ffmpegImageSink: resizing image to " << w << "x" << h << std::flush ;
    if (!resizeImage(&copy, w, h)) std::cerr << " (failed)" ;
    std::cerr << std::endl ;
  }

  if (!_chrono.count()) _active = init(&copy) ;
  if (!_active) return false ;

  // --------------------------------------------

  AVFrame frame ;
  frame.data[0] = copy.getData() ;
  int size = w*h ;
  frame.data[1] = frame.data[0] + size ;
  frame.data[2] = frame.data[1] + size / 4 ;
  frame.linesize[0] = w ;
  frame.linesize[1] = w / 2 ;
  frame.linesize[2] = w / 2 ;
  frame.quality = (int)_st.quality ;

  int nb = avcodec_encode_video(&(_st.codec), _outbuffer, sizeof(_outbuffer), &frame) ;
  _avfc.oformat->write_packet(&_avfc, 0, _outbuffer, nb, 0) ;

  _chrono.tick() ;
  return true ;
}

bool
ffmpegImageSink::stop(void) {
  bool result = false ;
  if (_active) {
    avcodec_close(&(_st.codec)) ;
    _avfc.oformat->write_trailer(&_avfc) ;
    _chrono.stop() ;
    _active = false ;
    result = true ;
  }
  if (_ownfd && _fd!=-1) close(_fd) ;
  _fd = -1 ;
  return result ;
}
