#ifndef n_ffmpegImageSource_H_
#define n_ffmpegImageSource_H_

/*
 *
 * nucleo/plugins/ffmpeg/ffmpegImageSource.H --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/core/TimeKeeper.H>
#include <nucleo/image/source/ImageSource.H>
#include <nucleo/core/URL.H>

#include <avformat.h>

using namespace nucleo ;

class ffmpegImageSource : public ImageSource {

private:

  Image::Encoding _encoding ;
  std::string _filename ;
  int _fd ;
  std::string _format ;

  int _length ;
  unsigned long _pace ;
  TimeKeeper *_tk ;

  AVCodec *_codec ;
  AVCodecContext *_codec_context ;
  uint8_t *_buffer ;
  int _bufbegin, _bufend ;
  AVFrame _frame ;

  enum {CLOSED, WAITING, READY } _state ;

protected:

  void react(Observable *) ;

public:

  ffmpegImageSource(Image::Encoding e, const URL &url, const char *format) ;

  virtual ~ffmpegImageSource(void) {
    stop() ; 
    delete [] _buffer ;
  }

  bool start(void) ;
  bool getNextImage(Image *img) ;
  bool isActive(void) { return (_state!=CLOSED) ; }
  bool stop(void) ;

} ;

#endif
