/*
 *
 * nucleo/plugins/howl/howlKeeper.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/plugins/howl/howlKeeper.H>

#include <iostream>

namespace nucleo {

  // ----------------------------------------------------------

  howlKeeper::howlKeeper(void) {
    // std::cerr << "Creating howlKeeper " << this << std::endl ;

    if (sw_rendezvous_init(&session) != SW_OKAY)
	 throw std::runtime_error("sw_rendezvous_init() failed") ;
    if (sw_rendezvous_salt(session, &salt) != SW_OKAY)
	 throw std::runtime_error("sw_rendezvous_salt failed") ;

    if (! ReactiveEngine::useNativeEngine(true)) {
	 // std::cerr << "howlKeeper: setting up timer" << std::endl ;
	 _tk = TimeKeeper::create(500*TimeKeeper::millisecond, true) ;
	 subscribeTo(_tk) ;
    } else {
	 _tk = 0 ;
    }
  }

  void
  howlKeeper::react(Observable *) {
    // std::cerr << "howlKeeper (" << this << "): react" << std::endl ;
    sw_ulong milliseconds = 75 ;
    sw_salt_step(salt, &milliseconds) ;
  }

  howlKeeper::~howlKeeper(void) {
    // std::cerr << "Deleting howlKeeper " << this << std::endl ;
    sw_rendezvous_fina(session) ;
    if (_tk) unsubscribeFrom(_tk) ;
    delete _tk ;
  }

  // ----------------------------------------------------------

  int howlKeeper::nbClients = 0 ;
  howlKeeper *howlKeeper::keeper = 0 ;

  void
  howlKeeper::attach(void) {
    nbClients++ ;
    if (!keeper) keeper = new howlKeeper() ;
  }

  sw_rendezvous
  howlKeeper::getSession(void) {
    return keeper ? (keeper->session) : 0 ;
  }

  void
  howlKeeper::detach(void) {
    nbClients-- ;
    if (!nbClients) {
	 delete keeper ;
	 keeper = 0 ;
    }
  }

  // ----------------------------------------------------------

}
