/*
 *
 * nucleo/plugins/howl/howlServiceFinder.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/plugins/howl/howlServiceFinder.H>

#include <rendezvous/text_record.h>

#include <iostream>

extern "C" {
  nucleo::howlServiceFinder*
  howlServiceFinder_factory(const char *type, nucleo::ServiceFinder::EventQueue *queue, const char *domain) {
    return new nucleo::howlServiceFinder(type,queue,domain) ;
  }
}

namespace nucleo {

  sw_result
  howlServiceFinder::sfResolver(sw_rendezvous_resolve_handler instance,
						  sw_rendezvous rendezvous,
						  sw_rendezvous_resolve_id id,
						  sw_const_string name,
						  sw_const_string type,
						  sw_const_string domain,
						  sw_ipv4_address address,
						  sw_port port,
						  sw_const_string text_record_string,
						  sw_octets text_record,
						  sw_ulong text_record_len,
						  sw_opaque extra) {
    sw_rendezvous_stop_resolve(rendezvous, id) ;

    howlServiceFinder *obj = (howlServiceFinder*)instance ;

    obj->rids.remove(id) ;

    Event *e = new Event ;
    e->event = Event::FOUND ;
    e->service.type = type ;
    e->service.name = name ;
    e->service.domain = domain ;
    char name_buf[16] ; sw_ipv4_address_name(address, name_buf, 16) ;
    e->service.ipaddr = name_buf ;
    e->service.port = port ;    

    sw_text_record_string_iterator	it1 ;
    sw_text_record_string_iterator_init(&it1, text_record_string) ;
    sw_char key[SW_TEXT_RECORD_MAX_LEN], value[SW_TEXT_RECORD_MAX_LEN] ;
    while (sw_text_record_string_iterator_next(it1, key, value) == SW_OKAY)
	 e->service.textRecords[key] = value ;
    sw_text_record_string_iterator_fina(it1) ;

    obj->queue->push(e) ;
    obj->notifyObservers() ;

    return SW_OKAY ;
  }

  sw_result
  howlServiceFinder::sfBrowseReply(sw_rendezvous_browse_handler instance,
							sw_rendezvous rendezvous,
							sw_rendezvous_browse_id id,
							sw_rendezvous_browse_status status,
							sw_const_string name,
							sw_const_string type,
							sw_const_string domain,
							sw_opaque extra) {

    howlServiceFinder *obj = (howlServiceFinder*)instance ;

    switch (status) {
    case SW_RENDEZVOUS_BROWSE_ADD_SERVICE: {
	 sw_rendezvous_resolve_id rid;
	 if (sw_rendezvous_resolve(rendezvous,
						  name, type, domain,
						  instance, sfResolver, NULL, &rid) != SW_OKAY)
	   std::cerr << "howlServiceFinder: sw_rendezvous_resolve failed" << std::endl ;
	 else
	   obj->rids.push_back(rid) ;
    } break;
    case SW_RENDEZVOUS_BROWSE_REMOVE_SERVICE: {
	 Event *e = new Event ;
	 e->event = Event::LOST ;
	 e->service.type = type ;
	 e->service.name = name ;
	 e->service.domain = domain ;
	 obj->queue->push(e) ;
	 obj->notifyObservers() ;
    } break;
    default:
	 break;
    }

    return SW_OKAY;
  }

  howlServiceFinder::howlServiceFinder(const char *type, EventQueue *q, const char *domain)
    : ServiceFinder(q) {
    howlKeeper::attach() ;
    if (sw_rendezvous_browse_services(howlKeeper::getSession(),
							   type, domain,
							   this, sfBrowseReply, NULL, &id) != SW_OKAY)
	 throw std::runtime_error("howlServiceFinder: sw_rendezvous_browse_services failed") ;
  }

  howlServiceFinder::~howlServiceFinder(void) {
    sw_rendezvous session = howlKeeper::getSession() ;
    sw_rendezvous_stop_browse_services(session, id) ;
    for (std::list<sw_rendezvous_resolve_id>::iterator i=rids.begin();
	    i!=rids.end();
	    ++i)
	 sw_rendezvous_stop_resolve(session, *i) ;
    howlKeeper::detach() ;
  }

}
