/*
 *
 * tests/test-SocketKeeper.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/core/ReactiveEngine.H>
#include <nucleo/core/FileKeeper.H>
#include <nucleo/network/tcp/TcpConnection.H>
#include <nucleo/utils/FileUtils.H>
#include <nucleo/utils/TimeUtils.H>

#include <unistd.h>

#include <iostream>
#include <sstream>
#include <stdexcept>

using namespace nucleo ;

class Reactor : public ReactiveObject {

protected:

  int _fd ;
  FileKeeper *_fk ;
  int _total ;
  Chronometer _chrono ;

  void react(Observable*) {
    _chrono.tick() ;
#if 1
    int size = getavail(_fd) ;
    char *buffer = new char [size] ;
    int bytesread = read(_fd, buffer, size) ;
    delete [] buffer ;
    _total += bytesread ;
    std::cerr << size << " - " << bytesread << " - " << _total << std::endl ;
    if (bytesread<1) {
	 std::cerr << _chrono.average() << " ticks/sec" << std::endl ;
	 ReactiveEngine::stop() ;
    }
#else
    if (_chrono.read()>10000) {
	 std::cerr << _chrono.average() << " ticks/sec" << std::endl ;
	 ReactiveEngine::stop() ;
    }
#endif
  }

public:

  Reactor(int fd, const char *request) {
    std::stringstream srequest ;
    srequest << "GET " << request << " HTTP/1.0\n\n" ;
    std::string s = srequest.str() ;
    write(fd, s.c_str(), s.size()) ;

    _total = 0 ;
    _fd = fd ;
    _fk = FileKeeper::create(fd, FileKeeper::R) ; 
    subscribeTo(_fk) ;

    _chrono.start() ;
  }

} ;

int
main(int argc, char **argv) {
  try {

   char *NATIVE = getenv("NUCLEO_USE_NATIVE_ENGINE") ;
   ReactiveEngine::useNativeEngine(!NATIVE || strcmp(NATIVE,"0")) ;

   char *HOST="localhost" ;
   int PORT=80 ;
   char *REQUEST="/" ;

   int test = (argc>1) ? atoi(argv[1]) : 0 ;

   switch (test) {
   case 0: HOST = "localhost" ; REQUEST = "/" ; break ;
   case 1: HOST = "insitu.lri.fr" ; REQUEST = "/~roussel/publications/dea-roussel.pdf" ; break ;
   case 2: HOST = "insitu.lri.fr" ; REQUEST = "/~roussel/publications/these-roussel.pdf" ; break ;
   case 3: HOST="spy.seas.columbia.edu" ; PORT = 5555 ; REQUEST = "/push/video" ; break ;
   }
   
   std::cerr << "http://" << HOST << ":" << PORT << REQUEST << std::endl ;

   TcpConnection c(HOST, PORT) ;
   Reactor r(c.getFd(), REQUEST) ;

   ReactiveEngine::run() ;

  } catch (std::runtime_error e) {
    std::cerr << "Runtime error: " << e.what() << std::endl ;
  } catch (std::exception e) {
    std::cerr << "Error: " << e.what() << std::endl ;
  }

  return 0 ;
}
