/*
 *
 * tests/test-ffmpeg-sink.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <assert.h>
#include <byteswap.h>
#ifndef be2me_32
#define be2me_32(x) bswap_32(x)
#endif
#include <avformat.h>

#include <nucleo/utils/AppUtils.H>
#include <nucleo/utils/SignalUtils.H>

#include <nucleo/image/source/ImageSource.H>
#include <nucleo/image/sink/ImageSink.H>

#include <nucleo/core/ReactiveEngine.H>

#include <string>
#include <iostream>
#include <stdexcept>

using namespace nucleo ;

bool signaled=false ;
void exitProc(int) { signaled = true ; }

int
main(int argc, char **argv) {
  try {
    char *SOURCE = 0 ;
    char *SINK = "file:output.mpg" ;

    if (parseCommandLine(argc, argv, "i:o:", "ss", &SOURCE, &SINK)<0) {
	 std::cerr << std::endl << argv[0] << " [-i source] [-o sink]" ;
	 std::cerr << std::endl ;
	 return 1 ;
    }

    if (!SOURCE) SOURCE = getenv("NUCLEO_SRC1") ;
    if (!SOURCE) SOURCE = "videoin:/anydev/anynode" ;
    ImageSource *source = ImageSource::create(Image::YpCbCr420, SOURCE) ;
    if (!source) {
	 std::cerr << "Couldn't create image source from " << SOURCE << std::endl ;
	 return -1 ;
    }

    Image img ;

    ImageSink *sink = ImageSink::create(SINK) ;

    // -----------------------------------------------------------------------

    std::cerr << "Available video formats (libavformat version " << LIBAVFORMAT_VERSION << "):" << std::endl ;
    for (AVOutputFormat *fmt=first_oformat; fmt; fmt=fmt->next) {
	 if (fmt->video_codec != CODEC_ID_NONE)
	   std::cerr << "  " << fmt->name << " (" << fmt->long_name << ")" << std::endl ;
    }
    std::cerr << std::endl ;

    // -----------------------------------------------------------------------

    trapAllSignals(exitProc) ;

    source->start() ;
    sink->start() ;

    while (!signaled && sink->isActive() && source->isActive()) {
	 ReactiveEngine::step() ;
	 if (source->getNextImage(&img))
	   std::cerr << (sink->handle(&img) ? "+" : "-") << std::flush ;
    }
    std::cerr << std::endl ;

    // -----------------------------------------------------------------------

    std::cerr << img.getWidth() << "x" << img.getHeight() << ", " << source->getFrameRate() << " frames/sec" << std::endl ;

  } catch (std::runtime_error e) {
    std::cerr << "Runtime error: " << e.what() << std::endl ;
  } catch (std::exception e) {
    std::cerr << "Error: " << e.what() << std::endl ;
  }

  return 0 ;
}
