/*
 *
 * tests/test-glWindow.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/core/ReactiveEngine.H>
#include <nucleo/utils/AppUtils.H>
#include <nucleo/gl/window/glWindow.H>

#include <iostream>
#include <string>

using namespace nucleo ;

bool FULLSCREEN = false ;

bool
debugEvents(std::string msg, glWindow *window) {
  bool refresh = false ;

  glWindow::event e ;
  while (window->getNextEvent(&e)) {
    std::cerr << msg ;
    switch( e.type ) {
    case glWindow::event::configure:
	 std::cerr << "configure: " << e.width << "x" << e.height << " " << e.x << "," << e.y << std::endl ;
	 window->makeCurrent() ;
	 glViewport(0,0,e.width, e.height) ;
	 refresh = true ;
	 break ;
    case glWindow::event::expose:
	 std::cerr << "expose" << std::endl ;
	 refresh = true ;
	 break ;
    case glWindow::event::destroy:
	 std::cerr << "destroy" << std::endl ;
	 return false ;
	 break ;
    case glWindow::event::enter:
	 std::cerr << "enter" << std::endl ;
	 break ;
    case glWindow::event::leave:
	 std::cerr << "leave" << std::endl ;
	 break ;
    case glWindow::event::keyPress:
	 std::cerr << "keyPress: " << e.keysym << " (" << e.keystr << ")" << std::endl ;
	 break ;
    case glWindow::event::keyRelease:
	 std::cerr << "keyRelease: " << e.keysym << " (" << e.keystr << ")" << std::endl ;
	 switch(e.keysym) {
	 case XK_Escape: return false ;
	 case XK_f: window->fullScreenMode(true) ; break ;
	 case XK_1: window->warpCursor(0,0) ; break ;
	 case XK_2: window->warpCursor(10,10) ; break ;
	 case XK_3: window->warpCursor(20,20) ; break ;
	 case XK_g:
	   window->fullScreenMode(false) ;
	   if (FULLSCREEN)
	   {
	     /* we started fullscreen, the first time we desactivate this
	      * state we must resize ourself */ 
	     FULLSCREEN = false;
	     window->resize(160,120);
	   }
	   break ;
	 default: break ;
	 }
	 break ;
    case glWindow::event::buttonPress:
	 std::cerr << "buttonPress: " << e.button << " " << e.x << "," << e.y << std::endl ;
	 break ;
    case glWindow::event::buttonRelease:
	 std::cerr << "buttonRelease: " << e.button << " " << e.x << "," << e.y << std::endl ;
	 break ;
    case glWindow::event::pointerMotion:
         std::cerr << "pointerMotion: " << e.x << "," << e.y << std::endl ;
	 break ;
    case glWindow::event::wheelMotion:
	 std::cerr << "wheelMotion: " << e.axis << "," << e.delta << std::endl ;
	 break ;
    default: break ;
    }
  }

  if (refresh) {
    std::cerr << "REFRESH "<< std::endl ;
    window->makeCurrent() ;
    glClear(GL_COLOR_BUFFER_BIT) ;
    glColor3f(1.0,1.0,0) ;
    glRectf(-0.5,-0.5,0.5,0.5) ;
    window->swapBuffers() ;
  }

  return true ;
}

int
main(int argc, char **argv)
{
  bool USE_TWO_WINDOWS = false ;
  if (parseCommandLine(argc, argv, "2f", "bb",
				   &USE_TWO_WINDOWS, &FULLSCREEN)<0) {
    std::cerr << std::endl << argv[0] << " [-f(ull-screen)] [-2 (windows)]" << std::endl ;
    exit(1) ;
  }

  long options = glWindow::DOUBLE_BUFFER ;
  long eventmask = glWindow::event::configure
    | glWindow::event::expose
    | glWindow::event::destroy
    | glWindow::event::enter
    | glWindow::event::leave
    | glWindow::event::keyPress
    | glWindow::event::keyRelease
    | glWindow::event::buttonPress
    | glWindow::event::buttonRelease
    | glWindow::event::pointerMotion
    | glWindow::event::wheelMotion ;

  glWindow *window1 = glWindow::create(options, eventmask) ;
  if (FULLSCREEN) window1->fullScreenMode(true) ;
  else window1->resize(160,120) ;
  window1->setTitle("#1") ;
  window1->map() ;
  window1->makeCurrent();
  glClearColor(0.1,0.5,0.4,1) ;

  glWindow *window2=0 ;
  if (USE_TWO_WINDOWS) {
    window2 = glWindow::create(options, eventmask) ;
    window2->resize(160,120) ;
    window2->setTitle("#2") ;
    window2->map() ;
    window2->makeCurrent();
    glClearColor(0.4,0.5,0.1,1) ;
   }

  glWindow::event e ;
  std::cerr << "e: " << e.keysym << " (" << e.keystr << ")" << std::endl ;

  for (bool loop=true; loop;) {
    ReactiveEngine::step() ;
    loop = loop && debugEvents("#1 ", window1) ;
    if (USE_TWO_WINDOWS) loop = loop && debugEvents("#2 ", window2) ;
  }

  return 0 ;
}
