/*
 *
 * tests/test-rdv.cxx --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <nucleo/utils/AppUtils.H>
#include <nucleo/utils/SignalUtils.H>

#include <nucleo/network/rdv/Service.H>
#include <nucleo/network/rdv/ServiceAnnouncer.H>
#include <nucleo/network/rdv/ServiceFinder.H>

#include <nucleo/core/ReactiveEngine.H>

#include <iostream>
#include <stdexcept>

using namespace nucleo ;

void exitProc(int s) { ReactiveEngine::stop() ; }

class Reactor : public ReactiveObject {

private:

  ServiceFinder::EventQueue *q ;

protected:

  void react(Observable*) {
    if (q->size()) {
	 ServiceFinder::Event *event = q->front() ;
	 q->pop() ;
	 event->debug(std::cerr) ;
	 delete event ;
    }
  }

public:

  Reactor(ServiceFinder *finder, ServiceFinder::EventQueue *queue) {
    q = queue ;
    subscribeTo(finder) ;
  }

} ;

int
main(int argc, char **argv) {
  char *NATIVE = getenv("NUCLEO_USE_NATIVE_ENGINE") ;
  ReactiveEngine::useNativeEngine(!NATIVE || strcmp(NATIVE,"0")) ;

  char *NAME = "nucleo" ;
  char *TYPE = "_http._tcp." ;
  int PORT = 8079 ;

  int firstArg = parseCommandLine(argc, argv, "n:t:p:", "ssi", &NAME, &TYPE, &PORT) ;
  if (firstArg<0) {
    std::cerr << std::endl << argv[0] << " [-n name] [-t type] [-p port]" << std::endl ;
    exit(1) ;
  }

  TextRecords textRecords ;
  for (int i=firstArg; i<argc-1; i+=2)
    textRecords[argv[i]] = argv[i+1] ;

  try {

#define ANNOUCE 1

#if ANNOUCE
    ServiceAnnouncer *announcer = ServiceAnnouncer::create(NAME, TYPE, PORT, &textRecords) ;
    if (!announcer) throw std::runtime_error("Unable to create ServiceAnnouncer") ;
#endif

    ServiceFinder::EventQueue q ;
    ServiceFinder *finder = ServiceFinder::create(TYPE, &q) ;
    if (!finder) throw std::runtime_error("Unable to create ServiceFinder") ;

    Reactor r(finder, &q) ;

    trapAllSignals(exitProc) ;
    ReactiveEngine::run() ;

#if ANNOUCE
    delete announcer ;
#endif
    delete finder ;

  } catch (std::runtime_error e) {
    std::cerr << "Runtime error: " << e.what() << std::endl ;
  } catch (...) {
    std::cerr << "Unknown error" << std::endl ;
  }

  return 0 ;
}
